import httpx
class Configurations:
    
    def verifyToken(ai_token: str):
        url = f"https://stg-gateway.quinnox.info:8243/authentication/v1/api/validateAPIToken?apiToken={ai_token}"
        with httpx.Client(timeout=600) as client:
            response = client.get(
                url = url,
                headers = {
                    "Authorization": "Bearer 90540897-748a-3ef2-b3a3-c6f8f42022da"
                }
            )
            response.raise_for_status()
            data = response.json()
            # print("\nDATA for verify token : ", data)
            if data.get("message") == "Authentication Token Validated Successfully.":
                return True # ai token is correct
            else:
                raise Exception("401") # mostly tells ai token is wrong
    
    def getDefaultGateway(ai_token: str):
        url = f"https://stg-gateway.quinnox.info:8243/authentication/v1/api/authenticateAPIToken?apiToken={ai_token}"
        with httpx.Client(timeout=600) as client:
            response = client.get(url, headers={"Authorization": "Bearer 90540897-748a-3ef2-b3a3-c6f8f42022da"})
            response.raise_for_status()
            data = response.json()
            # print("\nDATA for gateway details :", data)
            if data.get("status", False):
                del data["uuid"]
                return data
            else:
                raise Exception("401")
            
        
    def getNovaContextPath(_from: str):
        return f"nova-sdk/v1/api/nova_{_from}"
    
    def getApiBuilderContextPath(_from: str):
        return f"api-builder-sdk/v1/api/{_from}"
    
    def getDataAmplifierContextPath():
        return f"synthetic-data-generator-sdk/v1/api/datagen"

    def getAPIAssertions(_from: str):
        return f"api-assertion-gpt-sdk/v1/api/assertion/{_from}"
    
    def getVisionNovaContextPath(_from:str):
        return f"vision-nova-sdk/v1/api/{_from}"



# from nova import AsyncNova
from typing import Optional
from .nova import AsyncNova, SyncNova
from .api_builder.api_builder import AsyncApiBuilder,SyncApiBuilder
from .data_amplifier.data_amplifier import AsyncDataAmplifier, SyncDataAmplifier
from .api_assertions import AsyncAssertions, SyncAssertions
from .vision_nova import Async_Vision_Nova, Sync_Vision_Nova


class AsyncQyrusAI:

    # def __init__(self, api_key: str, base_url: Optional[str] = None) -> None:
    def __init__(self, api_key: str) -> None:
        self.nova = AsyncNova(api_key)
        self.api_builder = AsyncApiBuilder(api_key)
        self.data_amplifier = AsyncDataAmplifier(api_key)
        self.api_assertions = AsyncAssertions(api_key)
        self.vision_nova = Async_Vision_Nova(api_key)


class SyncQyrusAI:

    # def __init__(self, api_key: str, base_url: Optional[str] = None) -> None:
    def __init__(self, api_key: str) -> None:
        self.nova = SyncNova(api_key)
        self.api_builder = SyncApiBuilder(api_key)
        self.data_amplifier = SyncDataAmplifier(api_key)
        self.api_assertions = SyncAssertions(api_key)
        self.vision_nova = Sync_Vision_Nova(api_key)



# from _rest import AsyncHTTPClient, SyncHTTPClient
import asyncio
import os
from typing import Optional
from qyrusai._rest import AsyncHTTPClient, SyncHTTPClient
from qyrusai.configs import Configurations
from qyrusai._types import CreateScenariosResponse, JiraDetails
from urllib.parse import urljoin


class AsyncNovaJira:

    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")

    async def create(self, jira_endpoint: str,
                                      jira_api_token: str, jira_username: str,
                                      jira_id: str) -> CreateScenariosResponse:
        
        """Creates Test Scenarios using a JIRA ID
        Connects to a JIRA instance using provided credentials and domain URL, then generates test scenarios based on the specified JIRA Ticket ID.

        Args:
            jira_details (JiraDetails): A model containing the parameters
                required for connecting to JIRA.

        Returns:
            CreateScenariosResponse: A model representing the response of the operation with
                the success status, message, and a list of scenarios if successful.
        """
        # print(jira_api_token, jira_endpoint, jira_id, jira_username)
        
        # Join the base URL and the context path
        url = urljoin(self.base_url,Configurations.getNovaContextPath("jira"))

        data = {
            "jira_endpoint": jira_endpoint,
            "jira_api_token": jira_api_token,
            "jira_username": jira_username,
            "jira_id": jira_id
        }
        
        # JiraDetails.validate(data)
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f"Bearer "+self.gateway_token,
            'Custom': self.api_key
        }

        async_client = AsyncHTTPClient()
        response_data = await async_client.post(url, data, headers)
        # print(f"response_data (createScenariosFromJira) : {response_data}")
        # print(
        #     f"response_data (createScenariosFromJira) TYPE == >> : {type(response_data)}"
        # )

        return CreateScenariosResponse(**response_data)
        
class SyncNovaJira:

    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")

    def create(self, jira_endpoint: str,
                                      jira_api_token: str, jira_username: str,
                                      jira_id: str) -> CreateScenariosResponse:
        """Creates Test Scenarios using a JIRA ID
        Connects to a JIRA instance using provided credentials and domain URL, then generates test scenarios based on the specified JIRA Ticket ID.

        Args:
            jira_details (JiraDetails): A model containing the parameters
                required for connecting to JIRA.

        Returns:
            CreateScenariosResponse: A model representing the response of the operation with
                the success status, message, and a list of scenarios if successful.
        """
        # print(jira_api_token, jira_endpoint, jira_id, jira_username)
        
        # Join the base URL and the context path
        url = urljoin(self.base_url,Configurations.getNovaContextPath("jira"))

        data = {
            "jira_endpoint": jira_endpoint,
            "jira_api_token": jira_api_token,
            "jira_username": jira_username,
            "jira_id": jira_id
        }
        
        # JiraDetails.validate(data)
        headers = {
            'Content-Type': 'application/json',
            'Authorization': "Bearer "+self.gateway_token,
            'Custom': self.api_key
        }  # Headers if required

        sync_client = SyncHTTPClient()
        response_data = sync_client.post(url, data, headers)
        # print(f"response_data (createScenariosFromJira) : {response_data}")
        # print(
        #     f"response_data (createScenariosFromJira) TYPE == >> : {type(response_data)}"
        # )

        return CreateScenariosResponse(**response_data)


class AsyncNovaUserDescription:

    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")

    async def create(self, user_description: str) -> CreateScenariosResponse:
        """Creates Test Scenarios using a User Description.
        
        Args:
            user_description (str): A description by user to create test scenarios.

        Returns:
             CreateScenariosResponse: A model representing the response of the operation with
                the success status, message, and a list of scenarios if successful.
        """

        url = urljoin(self.base_url,Configurations.getNovaContextPath("user"))
        
        data = {
            "user_description": user_description,
        }
        
        headers = {
            'Content-Type': 'application/json',
            "Authorization": "Bearer "+self.gateway_token,
            "Custom": self.api_key
        }  
        
        async_client = AsyncHTTPClient()
        response_data = await async_client.post(url, data, headers)
        # print("📁.....", response_data)
        return CreateScenariosResponse(**response_data)
        
class SyncNovaUserDescription:
    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")


    def create(self, user_description: str) -> CreateScenariosResponse:
        """Creates Test Scenarios using a User Description.
        
        Args:
            user_description (str): A description by user to create test scenarios.

        Returns:
            CreateScenariosResponse: A model representing the response of the operation with
                the success status, message, and a list of scenarios if successful.
        """

        url = urljoin(self.base_url,Configurations.getNovaContextPath("user"))
        
        data = {
            "user_description": user_description,
        }
        
        headers = {
            'Content-Type': 'application/json',
            "Custom": self.api_key,
            "Authorization": "Bearer "+self.gateway_token
        }  # Headers if required
        sync_client = SyncHTTPClient()
        response_data = sync_client.post(url, data, headers)
        # print("📁.....", response_data)
        return CreateScenariosResponse(**response_data)



from pydantic import BaseModel, Field
from typing import List, Optional, Dict, Union


# Pydantic for Input and Response
class Scenario(BaseModel):
    test_script_name: str
    test_script_objective: str
    reason_to_test: str
    criticality_description: str
    criticality_score: int


class CreateScenariosResponse(BaseModel):
    ok: bool
    message: str
    scenarios: Optional[List[Scenario]] = None


class JiraDetails(BaseModel):
    jira_endpoint: str
    jira_api_token: str
    jira_username: str
    jira_id: str
    
class UserDescription(BaseModel):
    user_description: str
    
class ApiBuilderResponse(BaseModel):
    swagger_dictionary: dict
    
class DataAmplifierResponse(BaseModel):
    data: Optional[Dict] = None
    status: bool
    message: str
    
class AssertionHeaderRequest(BaseModel):
    headers: Union[List, Dict, str]
    
# class AssertionHeaderResponse(BaseModel):
#     assertions: Union[List, Dict, str]
class AssertionResponseRequest(BaseModel):
    response: Union[List, Dict, str]
    
    
class AssertionAllRequest(BaseModel):
    headers: Union[List, Dict, str]
    response: Union[List, Dict, str]


from typing import Optional
from .headers import AsyncHeaderAssertions,SyncHeaderAssertions
from .json_body import AsyncJSONBodyAssertions,SyncJSONBodyAssertions
from .json_path import AsyncJSONPathAssertions,SyncJSONPathAssertions
from .json_schema import AsyncJSONSchemaAssertions,SyncJSONSchemaAssertions
from .all_assertions import AsyncAllAssertions,SyncAllAssertions

class AsyncAssertions:
    def __init__(self, api_key: str) -> None:
        self.headers = AsyncHeaderAssertions(api_key)
        self.jsonbody = AsyncJSONBodyAssertions(api_key)
        self.jsonpath = AsyncJSONPathAssertions(api_key)
        self.jsonschema = AsyncJSONSchemaAssertions(api_key)
        self.all_assertions = AsyncAllAssertions(api_key)
        
class SyncAssertions:
    def __init__(self, api_key: str) -> None:
        self.headers = SyncHeaderAssertions(api_key)
        self.jsonbody = SyncJSONBodyAssertions(api_key)
        self.jsonpath = SyncJSONPathAssertions(api_key)
        self.jsonschema = SyncJSONSchemaAssertions(api_key)
        self.all_assertions = SyncAllAssertions(api_key)


# from nova.nova import AsyncNovaJira, AsyncNovaUserDescription
# from nova.nova import AsyncNovaJira, AsyncNovaUserDescription
from typing import Optional
from .nova import AsyncNovaJira, AsyncNovaUserDescription, SyncNovaJira, SyncNovaUserDescription


class AsyncNova:
    # def __init__(self, api_key: str, base_url: Optional[str] = None) -> None:
    def __init__(self, api_key: str) -> None:
        self.from_jira = AsyncNovaJira(api_key)
        self.from_description = AsyncNovaUserDescription(api_key)
        
class SyncNova:
    # def __init__(self, api_key: str, base_url: Optional[str] = None) -> None:
    def __init__(self, api_key: str) -> None:
        self.from_jira = SyncNovaJira(api_key)
        self.from_description = SyncNovaUserDescription(api_key)




from qyrusai.configs import Configurations
from typing import Optional
from urllib.parse import urljoin
from qyrusai._rest import AsyncHTTPClient,SyncHTTPClient
from qyrusai._types import AssertionAllRequest


class AsyncAllAssertions:
    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")
            
    async def create(self, headers:str, response:str):
        """Generate Header, JSON Assertions.

        Args:
            response (AssertionResponseRequest): response

        Returns:
            _type_: Returns the list of JSON key ,value and desriptions.
        """
        
        url = urljoin(self.base_url,Configurations.getAPIAssertions("all"))
        
        data = {
            "headers": headers,
            "response": response
        }
        
        headers = {
            'Content-Type': 'application/json',
            'Authorization': "Bearer "+self.gateway_token,
            'Custom': self.api_key
        }
        
        async_client = AsyncHTTPClient()
        response_data = await async_client.post(url, data, headers)

        
        # print(f"response_data [ASYNC] (ALL Assertions) : {response_data}")
        # print(
        #     f"response_data [ASYNC] (ALL Assertions) TYPE == >> : {type(response_data)}"
        # )
        
        return response_data
    
class SyncAllAssertions:
    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")
            
    def create(self, headers:str, response:str):
        """Generate Header, JSON Assertions.

        Args:
            response (AssertionResponseRequest): response

        Returns:
            _type_: Returns the list of header and JSON assertions
        """
        
        url = urljoin(self.base_url,Configurations.getAPIAssertions("all"))
        
        data = {
            "headers": headers,
            "response": response
        }
        
        headers = {
            'Content-Type': 'application/json',
            'Authorization': "Bearer "+self.gateway_token,
            'Custom': self.api_key
        }
        
        sync_client = SyncHTTPClient()
        response_data = sync_client.post(url, data, headers)

        
        # print(f"response_data [SYNC] (ALL Assertions) : {response_data}")
        # print(
        #     f"response_data [SYNC] (ALL Assertions) TYPE == >> : {type(response_data)}"
        # )
        
        return response_data


# from configs import Configurations
from qyrusai.configs import Configurations
from qyrusai._types import DataAmplifierResponse
from urllib.parse import urljoin
from qyrusai._rest import AsyncHTTPClient, SyncHTTPClient

class AsyncDataAmplifier:
    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")
            
    async def amplify(self, data, data_count):
        """To amplify your tabular data.

        Args:
            data (list): Data to be amplified.
            data_count (str): Give a slider value.
        
        Returns:
            DataAmplifierResponse: List of dictionaries representing the created/amplified data.
        """
        
        url = urljoin(self.base_url,Configurations.getDataAmplifierContextPath())
        
        body = {
            "data": data,
            "data_count": data_count,
        }
        
        headers = {
            'Content-Type': 'application/json',
            'Authorization': "Bearer "+self.gateway_token,
            'Custom': self.api_key
        }
        
        async_client = AsyncHTTPClient()
        
        response_data = await async_client.post(url, body, headers)
        
        # print(f"response_data (ASYNC-DA) : {response_data}")
        # print(
        #     f"response_data (ASYNC-DA) TYPE == >> : {type(response_data)}"
        # )
        
        return DataAmplifierResponse(**response_data)
        
class SyncDataAmplifier:
    def __init__(self, api_key: str):
        self.api_key = api_key
        # token_valid = asyncio.run(Configurations.verifyToken(api_key)) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        token_valid = Configurations.verifyToken(api_key) # If True is stored, well and good otherwise an Exception will have been raised already if something goes wrong.
        if not token_valid:
            raise Exception("401")
        gatewayDetails = Configurations.getDefaultGateway(api_key)
        # gatewayDetails = asyncio.run(Configurations.getDefaultGateway(api_key))
        self.base_url = gatewayDetails.get("gatewayUrl")
        self.gateway_token = gatewayDetails.get("gatewayToken")
            
    def amplify(self, data, data_count):
        """To amplify your tabular data.

        Args:
            data (list): Data to be amplified.
            data_count (str): Give a slider value.
        
        Returns:
            DataAmplifierResponse: List of dictionaries representing the created/amplified data.
        """
        
        url = urljoin(self.base_url,Configurations.getDataAmplifierContextPath())
        
        body = {
            "data": data,
            "data_count": data_count,
        }
        
        headers = {
            'Content-Type': 'application/json',
            'Authorization': "Bearer "+self.gateway_token,
            'Custom': self.api_key
        }
        
        sync_client = SyncHTTPClient()
        
        response_data = sync_client.post(url, body, headers)
        
        # print(f"response_data (SYNC-DA) : {response_data}")
        # print(
        #     f"response_data (SYNC-DA) TYPE == >> : {type(response_data)}"
        # )
        
        return DataAmplifierResponse(**response_data)
        